// main.cpp

#include "stdafx.h"

#include <vector>
#include <conio.h>

#include "ICsProject.h"
#include "ICsFile.h"
#include "IAssembly.h"
#include "ICsParserInterface.h"
#include "PerfomanceStatistics.h"

#include "CommandLineOptions.h"
#include "FileManager.h"
#include "PerfomanceCounter.h"

//--------------------------------------------------------------
// command line options:
//
//
//
// -assemblies- - clear include-assemblies directory list
// -assemblies - include-assemblies directories
// -waitkey - wait key press
// -project - process one project
// -projects - process multiple projects
// -m macro - set macro
// -m- macro - reset macro
// -ib directory - input base directory
// -ob directory - output base directory
//--------------------------------------------------------------

int main ( int argc, char * argv[] )
{
	std::vector<ICsFile*> files;
	std::vector<IAssembly*> assemblies;
	std::set<std::string> macros;

	bool bProjects = false;
	bool bWaitKey = false;

	enum arg_flag_enum
	{
		zero,
		recursive,
	};
	arg_flag_enum arg_flag = zero;

	PerfomanceStatistics perfomanceStatistics;

	for ( int i = 1; i < argc; ++i )
	{
		if ( argv[i][0] == '-' )
		{
			if ( strcmp ( argv[i], "-waitkey" ) == 0 )
			{
				bWaitKey = true;
				continue;
			}

			if ( strcmp ( argv[i], "-r" ) == 0 )
			{
				arg_flag = recursive;
				continue;
			}

			if ( strcmp ( argv[i], "-m" ) == 0 ) // -m macro
			{
				++i;
				if ( i < argc )
					macros.insert ( argv[i] );
				arg_flag = zero;
				continue;
			}

			if ( strcmp ( argv[i], "-m-" ) == 0 ) // -m- macro
			{
				++i;
				if ( i < argc )
					macros.erase ( argv[i] );
				arg_flag = zero;
				continue;
			}

			if ( strcmp ( argv[i], "-project" ) == 0 )
			{
				bProjects = false;
				continue;
			}
			if ( strcmp ( argv[i], "-projects" ) == 0 )
			{
				bProjects = true;
				continue;
			}

			if ( strcmp ( argv[i], "-assemblies-" ) == 0 )
			{
				getCommandLineOptions().include_assemblies_paths.clear();
				continue;
			}
			if ( strcmp ( argv[i], "-assemblies" ) == 0 )
			{
				++i;
				if ( i != argc )
				{
					FileManager & fileman = getFileManager();

					std::string current_directory;
					if ( !fileman.getCurrentDirectory ( current_directory ) )
					{
						ASSERT(0);
						continue;
					}

					std::string include_path;
					if ( !ConcatPathFile
						( current_directory.c_str(), argv[i], include_path ) )
					{
						ASSERT(0);
						continue;
					}

					getCommandLineOptions().include_assemblies_paths.push_back ( include_path );
				}
				continue;
			}
			
			if ( strcmp ( argv[i], "-ib" ) == 0 )
			{
				++i;
				if ( i != argc )
				{
					std::string path = argv[i];
					FileManager::normalizePath ( path );
					getCommandLineOptions().input_base_path = path.c_str();
				}
				continue;
			}
			if ( strcmp ( argv[i], "-ob" ) == 0 )
			{
				++i;
				if ( i != argc )
				{
					std::string path = argv[i];
					FileManager::createDirectoryRecursive ( path.c_str() );
					FileManager::normalizePath ( path );
					getCommandLineOptions().output_base_path = path.c_str();
				}
				continue;
			}

			ASSERT(0);
		}

		if ( bProjects )
		{
			DirectoryIterator iter ( argv[i] );
			while ( PFile directory = iter.next() )
			{
				std::string project_path;
				if ( directory->getFullPath ( project_path ) )
				{
					collect_files ( project_path.c_str(), true, files );
					collect_assemblies ( project_path.c_str(), false, assemblies );
					process_project ( files, assemblies, macros, perfomanceStatistics );
					files.clear();
					assemblies.clear();
				}
				else
					ASSERT(0);
			}
			arg_flag = zero;
		}
		else
		{
			std::string project_path ( argv[i] );
			if ( FileManager::normalizePath ( project_path ) )
			{
				collect_files ( project_path.c_str(), true, files );
				collect_assemblies ( project_path.c_str(), false, assemblies );
				process_project ( files, assemblies, macros, perfomanceStatistics );
				files.clear();
				assemblies.clear();
			}
			else
				ASSERT(0);
			arg_flag = zero;
		}
	}

	printf ( "total file bytes      : %u\n", perfomanceStatistics.total_file_bytes_ );
	printf ( "total assembly bytes  : %u\n", perfomanceStatistics.total_assembly_bytes_ );

	printf ( "total parsing (pass 1)  : %s sec\n", perfomanceStatistics.pass1.getText().c_str() );
	printf ( "total parsing (pass 2)  : %s sec\n", perfomanceStatistics.pass2.getText().c_str() );
	printf ( "total parsing (pass 3)  : %s sec\n", perfomanceStatistics.pass3.getText().c_str() );
	printf ( "total parsing (pass 4)  : %s sec\n", perfomanceStatistics.pass4.getText().c_str() );
	printf ( "total parsing (metadata): %s sec\n", perfomanceStatistics.metadata_pass.getText().c_str() );

	printf ( "average bytes/sec (pass 1)  : %f\n", perfomanceStatistics.total_file_bytes_ / perfomanceStatistics.pass1.getValue() );
	printf ( "average bytes/sec (pass 2)  : %f\n", perfomanceStatistics.total_file_bytes_ / perfomanceStatistics.pass2.getValue() );
	printf ( "average bytes/sec (pass 3)  : %f\n", perfomanceStatistics.total_file_bytes_ / perfomanceStatistics.pass3.getValue() );
	printf ( "average bytes/sec (pass 4)  : %f\n", perfomanceStatistics.total_file_bytes_ / perfomanceStatistics.pass4.getValue() );
	printf ( "average bytes/sec (metadata): %f\n", perfomanceStatistics.total_assembly_bytes_ / perfomanceStatistics.metadata_pass.getValue() );
	printf ( "average bytes/sec           : %f\n", perfomanceStatistics.total_file_bytes_ /
	  ( perfomanceStatistics.pass1.getValue()
	  + perfomanceStatistics.pass2.getValue()
	  + perfomanceStatistics.pass3.getValue()
		+ perfomanceStatistics.pass4.getValue() ) );

	if ( bWaitKey )
		getch();

	return 0;
}

//--------------------------------------------------------------

void collect_files ( const char * path, bool recursive, std::vector<ICsFile*> & files )
{
	files.clear();
	FileSearchIterator file_iter ( path, "*.cs", recursive );
	while ( PFile file = file_iter.next() )
	{
		ICsFile * ifile = process_file ( file );
		if ( ifile )
			files.push_back ( ifile );
	}
}

//--------------------------------------------------------------

ICsFile * process_file ( PFile file ) 
{
	if ( !file->open ( File::ac_read ) )
		return 0;

	unsigned length;
	if ( !file->getLength ( length ) )
		return 0;
	
	char * buffer = (char*)ICsParserInterface::alloc ( length+1 );
	if ( !file->read ( buffer, length ) )
	{
		ICsParserInterface::free ( buffer );
		return 0;
	}
	buffer[length] = '\0';
	
	std::string path;
	file->getFullPath ( path );
	
	return ICsFile::create ( buffer, length, true, path.c_str() );
}

//--------------------------------------------------------------

void collect_assemblies ( const char * path, bool recursive, std::vector<IAssembly*> & assemblies )
{
	assemblies.clear();
	FileSearchIterator file_iter ( path, "*.dll", recursive );
	while ( PFile file = file_iter.next() )
	{
		IAssembly * iassembly = process_assembly ( file );
		if ( iassembly )
			assemblies.push_back ( iassembly );
	}
}

//--------------------------------------------------------------

IAssembly * process_assembly ( PFile file ) 
{
	if ( !file->open ( File::ac_read ) )
		return 0;

	unsigned length;
	if ( !file->getLength ( length ) )
		return 0;

	unsigned char * buffer = new unsigned char[length];
	if ( !file->read ( buffer, length ) )
	{
		delete[] buffer;
		return 0;
	}

	std::string path;
	file->getFullPath ( path );

	return IAssembly::create ( buffer, length, true, path.c_str() );
}

//--------------------------------------------------------------

CommandLineOptions & getCommandLineOptions()
{
	return CommandLineOptions::getInstance();
}

//--------------------------------------------------------------

CommandLineOptions & CommandLineOptions::getInstance()
{
	static CommandLineOptions instance;
	return instance;
}

//--------------------------------------------------------------
